/*
 * mySQLenum
 *  
 * Copyright (C) 2009 Andrea Fabrizi <andrea.fabrizi@gmail.com>
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <getopt.h>
#include <signal.h>
#include "mysqlenum.h"

/*
 * usage
 */
void usage(const char* program_name)
{

#ifndef BLIND
    fprintf(stderr, "mySQLenum %s\n"
                       " usage: %s [OPTION...]\n\n"
                       " OPTIONS:\n"
                       "   -c,   --config\t\t config file (default is mysqlenum.conf).\n"
                       "   -q,   --sql-query\t\t SQL Query to execute.\n"
                       "   -r,   --start-row\t\t starts the enumeration from this row (default is the  first row).\n"
                       "   -m,   --macro\t\t start macro mode.\n"         
                       
                       "\n"
                       "         --no-spinner\t\t disable the spinner.\n"
                       "         --quiet\t\t quiet (no output).\n"
                       "   -h,   --help\t\t\t this help.\n"
                       "   -V,   --version\t\t display the version and exit.\n"
                       "   -v,   --verbose\t\t be verbose (-vvv for very verbose).\n"
                       "\n"
                       
    , VERSION, program_name);
#endif    
    exit(EXIT_FAILURE);
}


/*
 * main
 */
int main(int argc, char *argv[])
{
    char *sqlquery = NULL,
         c,
         *config_file = "mysqlenum.conf";
    int startrow = -1;
    struct configuration conf;

    requests_count = 0;
    quiet = 0;
    nospinner = NO;
    mode_macro = 0;
    verbose = 0;
    bytes_sent = 0;
    bytes_received = 0;
    interrupt = 0;
    last_interrupt = 0;
        
    /* Arguments parsing */
    while (1)
    {
        static struct option long_options[] =
        {
            {"quiet", no_argument, &quiet, 1},
            {"sql-query",  required_argument, 0, 'q'},
            {"start-row",  required_argument, 0, 'r'},
            {"config",  required_argument, 0, 'c'},
            {"macro",  no_argument, 0, 'm'},
            {"no-spinner", no_argument, &nospinner, YES},

            {"verbose", no_argument, 0, 'v'},
            {"help",  no_argument, 0, 'h'},
            {"version",  no_argument, 0, 'V'},
            {0, 0, 0, 0}
        };
        
        int option_index = 0;

        c = getopt_long (argc, argv, "r:c:q:Vhvm", long_options, &option_index);
 
        if (c == -1)
            break;
       
        switch (c)
        {
            case 0:
                if (long_options[option_index].flag != 0)
                    break;
            break;

            /* query */
            case 'q':
                if (!str_empty(optarg))
                    sqlquery = optarg;
            break; 

            /* start row */
            case 'r':
                if (!str_empty(optarg))
                    startrow = atoi(optarg);
            break; 
            
            /* config file */
            case 'c':
                if (!str_empty(optarg) > 0)
                    config_file = optarg;
            break; 
                                   
            /* help */
            case 'h':
                usage(argv[0]);
            break;

            /* macro mode */
            case 'm':
                mode_macro = 1;
            break;
            
            /* verbose */
            case 'v':
                verbose++;
            break;
            
            /* version */
            case 'V':
#ifndef BLIND
                fprintf(stdout, "mySQLenum %s\n", VERSION);
#endif
                exit(EXIT_SUCCESS);
            break;
                                                            
            case '?':
                exit(EXIT_FAILURE);
            break;

            default:
                abort ();
        }
    }

   
    if (quiet)
        nospinner = YES;

#ifndef BLIND
    if (!quiet)
        fprintf(stdout, "mySQLenum %s\n"
                        "Copyright (C) 2009-2010 Andrea Fabrizi <andrea.fabrizi@gmail.com>\n"
                        
        , VERSION
        );
#endif

    /* Parsing config file... */
    if(!parse_config_file(config_file, &conf))
        return EXIT_FAILURE;

    /* Override sqlquery parameter */
    if (sqlquery != NULL)
        conf.sqlquery = sqlquery;

    
    /* Override startrow parameter */
    if (startrow >= 0)
        conf.startrow = startrow;
    
    signal (SIGINT, (void *)interrupt_enumeration);
    
    /* Starting macro mode enumeration */    
    if (mode_macro)
         macro_mode_start(&conf);

    /* Starting classic enumeration */
    else
    {
        if (!quiet)
            fprintf(stdout, "\n Query: %s\n\n", conf.sqlquery);
        
        if (conf.multirowsenum == YES)
            enum_query_all_rows(&conf);
        else
            enum_query(&conf,conf.sqlquery);
    }
 
    /* Print some stats */   
    if (!quiet)   
        fprintf(stdout, "\n > Total requests: %d"
                        "\n > Data sent: %d Kb"
                        "\n > Data received: %d Kb\n"
        , requests_count, bytes_sent/1024, bytes_received/1024);
 

    return EXIT_SUCCESS;
}
